package org.processmining.streaminductiveminer.algorithms.stream.readers;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.eventstream.readers.acceptingpetrinet.AcceptingPetriNetXSReaderResultVisualizer;
import org.processmining.eventstream.readers.acceptingpetrinet.XSEventStreamToAcceptingPetriNetReader;
import org.processmining.framework.packages.PackageManager.Canceller;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.InductiveMiner.dfgOnly.Dfg;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiner;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiningParameters;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.InvalidProcessTreeException;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.NotYetImplementedException;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.PetrinetWithMarkings;
import org.processmining.stream.core.enums.CommunicationType;
import org.processmining.stream.reader.views.XSReaderSequenceOfResultsView;
import org.processmining.streaminductiveminer.algorithms.stream.readers.abstr.AbstractStreamInductiveMiner;
import org.processmining.streaminductiveminer.parameters.StreamInductiveMinerParameters;

public class StreamInductiveMinerAPNXSEventReaderImpl extends AbstractStreamInductiveMiner<AcceptingPetriNet>
		implements XSEventStreamToAcceptingPetriNetReader {

	public StreamInductiveMinerAPNXSEventReaderImpl(final PluginContext context,
			final CommunicationType communicationType, final StreamInductiveMinerParameters params) {
		super("Stream Inductive Miner (Accepting Petri Net, CxA: "
				+ params.getCaseAdministrationDataStructureType().getName() + "-"
				+ params.getCaseAdministrationDataStructureParameters().toString() + "; AxA: "
				+ params.getInternalRepresentationDataStructureType().getName() + "-"
				+ params.getInternalRepresentationDataStructureParameters().toString() + ")",
				new XSReaderSequenceOfResultsView<>("Visualizer of: Stream Inductive Miner (Accepting Petri net)", null,
						params.getRefreshRate(), AcceptingPetriNetXSReaderResultVisualizer.instance(context)),
				communicationType, params);
		((XSReaderSequenceOfResultsView<AcceptingPetriNet>) getVisualization()).setReader(this);
	}

	protected AcceptingPetriNet discover(Dfg dfg, DfgMiningParameters dfgParams) {
		PetrinetWithMarkings pnwm;
		AcceptingPetriNet result = null;
		try {
			pnwm = ProcessTree2Petrinet.convert(DfgMiner.mine(dfg, dfgParams, new Canceller() {
				public boolean isCancelled() {
					return false;
				}
			}));
			result = AcceptingPetriNetFactory.createAcceptingPetriNet(pnwm.petrinet, pnwm.initialMarking,
					pnwm.finalMarking);
		} catch (NotYetImplementedException | InvalidProcessTreeException e) {
			e.printStackTrace();
		}
		return result;
	}
}
