package org.processmining.streaminductiveminer.algorithms.stream.readers.abstr;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.basicutils.parameters.PluginParameters;
import org.processmining.eventstream.readers.abstr.AbstractXSEventReader;
import org.processmining.framework.util.Pair;
import org.processmining.logabstractions.models.DirectlyFollowsAbstraction;
import org.processmining.plugins.InductiveMiner.dfgOnly.Dfg;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgImpl;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiningParameters;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiningParametersIMd;
import org.processmining.stream.core.enums.CommunicationType;
import org.processmining.stream.core.interfaces.XSVisualization;
import org.processmining.streamabstractrepresentation.algorithms.UncoupledXEventClassDFAReaderImpl;
import org.processmining.streamabstractrepresentation.parameters.UnCoupledAbstractionSchemeParametersImpl;
import org.processmining.streaminductiveminer.parameters.StreamInductiveMinerParameters;

public abstract class AbstractStreamInductiveMiner<R>
		extends AbstractXSEventReader<R, R, StreamInductiveMinerParameters> {

	private final UncoupledXEventClassDFAReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl> dfaDelegate;

	public AbstractStreamInductiveMiner(final String name, XSVisualization<R> visualization,
			final CommunicationType communicationType, final StreamInductiveMinerParameters params) {
		super(name, visualization, params);
		dfaDelegate = new UncoupledXEventClassDFAReaderImpl<Object, UnCoupledAbstractionSchemeParametersImpl>(
				"DFA Reader (Stream Inductive Miner)", params);
	}

	public R computeCurrentResult() {
		DirectlyFollowsAbstraction<XEventClass> dfa = dfaDelegate.getCurrentResult();
		Dfg dfg = new DfgImpl(dfa.getAllGEQThreshold().size());
		for (Pair<XEventClass, XEventClass> p : dfa.getAllGEQThreshold()) {
			dfg.addDirectlyFollowsEdge(p.getFirst(), p.getSecond(), (long) dfa.getValue(p.getFirst(), p.getSecond()));
		}
		DfgMiningParameters dfgParams = new DfgMiningParametersIMd();
		dfgParams.setDebug(getParameters().getMessageLevel() >= PluginParameters.DEBUG);
		return discover(dfg, dfgParams);
	}

	protected abstract R discover(Dfg dfg, DfgMiningParameters dfgParams);

	public void processNewXSEvent(String caseId, XEventClass activity) {
		dfaDelegate.processNewXSEvent(caseId, activity);
	}
}
