package org.processmining.streaminductiveminer.plugins;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.eventstream.core.interfaces.XSEventStream;
import org.processmining.eventstream.readers.processtree.XSEventStreamToProcessTreeReader;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.stream.core.enums.CommunicationType;
import org.processmining.streamabstractrepresentation.plugins.AbstractUncoupledStreamMinerDefaultDialogsPlugin;
import org.processmining.streaminductiveminer.algorithms.stream.readers.StreamInductiveMinerPTXSEventReaderImpl;
import org.processmining.streaminductiveminer.parameters.StreamInductiveMinerParameters;

@Plugin(name = "Discover Process Tree(s) (Inductive Miner)", parameterLabels = { "Event Stream",
		"Stream-Based Inductive Miner Parameters" }, returnLabels = {
				"Inductive Miner (Event Stream)" }, returnTypes = {
						XSEventStreamToProcessTreeReader.class }, userAccessible = true)
public class StreamInductiveMinerProcessTreePlugin extends
		AbstractUncoupledStreamMinerDefaultDialogsPlugin<XSEventStreamToProcessTreeReader, StreamInductiveMinerParameters> {

	@UITopiaVariant(author = "S.J. van Zelst", email = "s.j.v.zelst@tue.nl", affiliation = "Eindhoven University of Technology")
	@PluginVariant(requiredParameterLabels = { 0 })
	public XSEventStreamToProcessTreeReader apply(final UIPluginContext context, final XSEventStream stream) {
		return super.apply(context, stream);
	}

	@UITopiaVariant(author = "S.J. van Zelst", email = "s.j.v.zelst@tue.nl", affiliation = "Eindhoven University of Technology", uiLabel = UITopiaVariant.USEVARIANT)
	@PluginVariant(variantLabel = "Discover Process Tree(s) (Inductive Miner, Default Parameters)", requiredParameterLabels = {
			0 })
	public XSEventStreamToProcessTreeReader applyDefault(final PluginContext context, final XSEventStream stream) {
		return super.apply(context, stream, getFreshParameters(context));
	}

	protected XSEventStreamToProcessTreeReader constructReader(PluginContext context, CommunicationType comm,
			StreamInductiveMinerParameters parameters) {
		return new StreamInductiveMinerPTXSEventReaderImpl(context, comm, parameters);
	}

	protected StreamInductiveMinerParameters getFreshParameters(PluginContext context) {
		return new StreamInductiveMinerParameters();
	}

}
